"""
Santec Communication DLL Wrapper.
"""

import Santec.Communication as Comm

# Define the main classes and types for communication
GPIBConnectType = Comm.GPIBConnectType
MainCommunication = Comm.MainCommunication
CommunicationMethod = Comm.CommunicationMethod
